"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTenantDAO = void 0;
const cleanPatch_1 = __importDefault(require("@thatamplify/core/api/utils/cleanPatch"));
const DataStoreError_1 = __importDefault(require("@thatamplify/core/errors/DataStoreError"));
const useDebounce_1 = require("@thatreact/core-hooks/hooks/useDebounce");
const catchAndThrow_1 = __importDefault(require("@thats/utils/errors/catchAndThrow"));
const datastore_1 = require("aws-amplify/datastore");
const react_1 = require("react");
const ApplifyController_1 = require("../../controllers/ApplifyController");
const useDataStoreGet_1 = __importDefault(require("../../hooks/useDataStoreGet"));
const reg_1 = __importDefault(require("../reg"));
exports.default = useTenantDAO;
function useTenantDAO(id) {
    const tenantId = (0, ApplifyController_1.useApplifyTenantId)();
    const { _, isLoading } = (0, useDataStoreGet_1.default)(reg_1.default.Tenant, id);
    const patch = (0, react_1.useRef)(undefined);
    const cachedItem = (0, react_1.useRef)(undefined);
    const cachedItemVersion = (0, react_1.useRef)(0);
    const [cacheChanged, setCacheChanged] = (0, react_1.useState)(new Date());
    (0, react_1.useEffect)(() => {
        cachedItem.current = undefined;
        cachedItemVersion.current = 0;
        patch.current = undefined;
        setCacheChanged(new Date());
    }, [id]);
    const triggerDataStoreUpdate = (0, useDebounce_1.useDebounce1000)(() => __awaiter(this, void 0, void 0, function* () {
        const _patch = patch.current;
        patch.current = undefined;
        if (!_patch)
            return;
        const latestItem = (yield datastore_1.DataStore.query(reg_1.default.Tenant, (p) => p.tenantId.eq(tenantId) && p.id.eq(id)))[0];
        (0, cleanPatch_1.default)(_patch);
        yield datastore_1.DataStore.save(reg_1.default.Tenant.copyOf(latestItem, u => Object.assign(u, _patch))).catch((0, catchAndThrow_1.default)('updating Tenant', DataStoreError_1.default));
    }), [tenantId, id]);
    const updateTenant = (0, react_1.useCallback)((_patch) => {
        const alreadyHasAPatch = !!patch.current;
        patch.current = _patch;
        cachedItem.current = reg_1.default.Tenant.copyOf(cachedItem.current, u => {
            Object.assign(u, _patch);
        });
        if (!alreadyHasAPatch)
            cachedItemVersion.current++;
        setCacheChanged(new Date());
        triggerDataStoreUpdate();
    }, [triggerDataStoreUpdate]);
    (0, react_1.useEffect)(() => {
        var _a;
        if (!!patch.current)
            return;
        if ((cachedItemVersion === null || cachedItemVersion === void 0 ? void 0 : cachedItemVersion.current) >= (_ === null || _ === void 0 ? void 0 : _['_version']))
            return;
        cachedItem.current = _;
        cachedItemVersion.current = (_a = _ === null || _ === void 0 ? void 0 : _['_version']) !== null && _a !== void 0 ? _a : undefined;
        setCacheChanged(new Date());
    }, [_]);
    return {
        tenant: cachedItem.current,
        updateTenant,
        tenantReady: !isLoading && !!cachedItem.current,
    };
}
exports.useTenantDAO = useTenantDAO;
//# sourceMappingURL=useTenantDAO.js.map