import Tenant from '@thatamplify/core/app/Tenant/Tenant'
import TenantPS from '@thatamplify/core/app/_propsets/TenantPS'
import RecordType from '@thatamplify/core/constants/RecordType'
import { useEffect, useRef, useState } from 'react'
import { useApplifyTenantId } from '../../controllers/ApplifyController'
import { useDataStoreList } from '../../hooks/useDataStoreList'
import reg from '../reg'

export default useTenantsDAO

export type TenantPatch = { name: string; p: Partial<TenantPS<RecordType.FLAT>> }

export interface TenantsDAO {
	tenants: Tenant[]

	tenantsReady: boolean
}

export function useTenantsDAO(): TenantsDAO {
	const { _, isLoading } = useDataStoreList(reg.Tenant)

	const tenantId = useApplifyTenantId()
	const itemCache = useRef<Tenant[]>([])

	const [cacheChanged, setCacheChanged] = useState(new Date())

	useEffect(() => {
		// Only update if:
		// 1) the number of items returned has changed
		// OR
		// 2) an item with a LARGER version has been sent
		if (
			_?.length !== itemCache?.current.length ||
			_.some(item => {
				const cachedItem = itemCache.current.find(i => i.id === item.id)
				if (!cachedItem) return true
				const cv = cachedItem['_version']
				const iv = item['_version']
				if (cv === undefined && iv === 1) return false
				return iv > cv
			})
		) {
			itemCache.current = _
			setCacheChanged(new Date())
		}
	}, [_])

	return {
		tenants: itemCache.current,

		tenantsReady: !isLoading,
	}
}
