"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTenantUserInvitesDAO = void 0;
const cleanPatch_1 = __importDefault(require("@thatamplify/core/api/utils/cleanPatch"));
const DataStoreError_1 = __importDefault(require("@thatamplify/core/errors/DataStoreError"));
const useDebounce_1 = require("@thatreact/core-hooks/hooks/useDebounce");
const swap_1 = require("@thats/utils/array/swap");
const catchAndThrow_1 = __importDefault(require("@thats/utils/errors/catchAndThrow"));
const datastore_1 = require("aws-amplify/datastore");
const react_1 = require("react");
const ApplifyController_1 = require("../../controllers/ApplifyController");
const useDataStoreList_1 = require("../../hooks/useDataStoreList");
const reg_1 = __importDefault(require("../reg"));
exports.default = useTenantUserInvitesDAO;
function useTenantUserInvitesDAO() {
    const { _, isLoading } = (0, useDataStoreList_1.useDataStoreList)(reg_1.default.TenantUserInvite);
    const tenantId = (0, ApplifyController_1.useApplifyTenantId)();
    const itemCache = (0, react_1.useRef)([]);
    const [cacheChanged, setCacheChanged] = (0, react_1.useState)(new Date());
    const noOfPatches = (0, react_1.useRef)(0);
    const patches = (0, react_1.useRef)({});
    const triggerDataStoreUpdate = (0, useDebounce_1.useDebounce1000)(() => __awaiter(this, void 0, void 0, function* () {
        const p = patches.current;
        for (const id of Object.keys(p)) {
            const patchesBeingProcessed = [...p[id]];
            for (const patch of p[id]) {
                const latestItem = (yield datastore_1.DataStore.query(reg_1.default.TenantUserInvite, (p) => p.tenantId.eq(tenantId) && p.id.eq(id)))[0];
                (0, cleanPatch_1.default)(patch);
                yield datastore_1.DataStore.save(reg_1.default.TenantUserInvite.copyOf(latestItem, u => Object.assign(u, patch))).catch((0, catchAndThrow_1.default)('updating TenantUserInvite', DataStoreError_1.default));
                noOfPatches.current--;
            }
            p[id] = p[id].filter(pp => patchesBeingProcessed.indexOf(pp) < 0);
        }
    }), [tenantId]);
    const updateTenantUserInvite = (0, react_1.useCallback)((id, patch) => {
        const ic = itemCache.current;
        const idx = ic.findIndex(p => p.id === id);
        const original = ic[idx];
        const patched = reg_1.default.TenantUserInvite.copyOf(original, u => {
            Object.assign(u, patch);
            u['_version']++;
        });
        itemCache.current = (0, swap_1.swap)(ic, patched, x => x === original);
        setCacheChanged(new Date());
        const p = patches.current;
        if (!p[id])
            p[id] = [];
        p[id].push(patch);
        noOfPatches.current++;
        triggerDataStoreUpdate();
    }, [triggerDataStoreUpdate]);
    const updateTenantUserInvitePS = (0, react_1.useCallback)((id, patch) => {
        const ic = itemCache.current;
        const idx = ic.findIndex(p => p.id === id);
        const original = ic[idx];
        updateTenantUserInvite(id, Object.assign(Object.assign({}, original), { p: Object.assign(Object.assign({}, original.p), patch) }));
    }, [updateTenantUserInvite]);
    const itemsToDelete = (0, react_1.useRef)([]);
    const triggerDataStoreDelete = (0, useDebounce_1.useDebounce1000)(() => __awaiter(this, void 0, void 0, function* () {
        const items = itemsToDelete.current;
        for (const item of items) {
            const latestItem = (yield datastore_1.DataStore.query(reg_1.default.TenantUserInvite, (p) => p.tenantId.eq(tenantId) && p.id.eq(item.id)))[0];
            yield datastore_1.DataStore.delete(latestItem).catch((0, catchAndThrow_1.default)('deleting TenantUserInvite', DataStoreError_1.default));
            itemsToDelete.current = itemsToDelete.current.filter(i => i.id !== item.id);
        }
    }), [tenantId]);
    const deleteTenantUserInvite = (0, react_1.useCallback)((itemOrItems) => {
        const items = Array.isArray(itemOrItems) ? itemOrItems : [itemOrItems];
        for (const item of items) {
            itemCache.current = itemCache.current.filter(i => i.id !== item.id);
            setCacheChanged(new Date());
            itemsToDelete.current.push(item);
        }
        triggerDataStoreDelete();
    }, [triggerDataStoreDelete]);
    (0, react_1.useEffect)(() => {
        if (noOfPatches.current > 0)
            return;
        if (itemsToDelete.current.length > 0)
            return;
        if ((_ === null || _ === void 0 ? void 0 : _.length) !== (itemCache === null || itemCache === void 0 ? void 0 : itemCache.current.length) ||
            _.some(item => {
                const cachedItem = itemCache.current.find(i => i.id === item.id);
                if (!cachedItem)
                    return true;
                const cv = cachedItem['_version'];
                const iv = item['_version'];
                if (cv === undefined && iv === 1)
                    return false;
                return iv > cv;
            })) {
            itemCache.current = _;
            setCacheChanged(new Date());
        }
    }, [_]);
    return {
        tenantUserInvites: itemCache.current,
        updateTenantUserInvite,
        updateTenantUserInvitePS,
        deleteTenantUserInvite,
        tenantUserInvitesReady: !isLoading,
    };
}
exports.useTenantUserInvitesDAO = useTenantUserInvitesDAO;
//# sourceMappingURL=useTenantUserInvitesDAO.js.map