import React, { useMemo } from 'react'
import {
	useApplifyAuthenticated,
	useApplifyInitialized,
	useApplifyTenantId,
} from '../controllers/ApplifyController'

export interface AuthAppSwitchProps {
	AuthApp: React.FunctionComponent
	NoauthApp: React.FunctionComponent
	AuthNoTenantApp?: React.FunctionComponent
}

export function AuthAppSwitch(props: AuthAppSwitchProps) {
	const { AuthApp, AuthNoTenantApp, NoauthApp } = props
	const tenantId = useApplifyTenantId()
	const authenticated = useApplifyAuthenticated()
	const initialized = useApplifyInitialized()

	return useMemo(() => {
		if (!initialized) return null
		if (!!AuthNoTenantApp && authenticated && !tenantId) {
			return <AuthNoTenantApp />
		} else if (authenticated) {
			return <AuthApp />
		} else {
			return <NoauthApp />
		}
	}, [AuthNoTenantApp, initialized, authenticated, tenantId, AuthApp, NoauthApp])
}
export default AuthAppSwitch
