import RolesManager from '@thatamplify/core/auth/classes/RolesManager'
import ApplifyEnv from '@thatamplify/core/env/ApplifyEnv'
import useCtrlActionsCtx from '@thatreact/ctrl/hooks/useCtrlActionsCtx'
import useCtrlCtx from '@thatreact/ctrl/hooks/useCtrlCtx'
import useCtrlState from '@thatreact/ctrl/hooks/useCtrlState'
import useCtrlStateSetter from '@thatreact/ctrl/hooks/useCtrlStateSetter'
import CtrlState from '@thatreact/ctrl/types/CtrlState'
import { SignUpOutput } from 'aws-amplify/auth'
import { DateTime } from 'luxon'
import React, { MutableRefObject, useContext, useMemo, useRef } from 'react'
import useApplifyDoClearDb from './actions/useApplifyDoClearDb'
import useApplifyDoConfirmSignIn from './actions/useApplifyDoConfirmSignIn'
import useApplifyDoConfirmSignUp from './actions/useApplifyDoConfirmSignUp'
import useApplifyDoDataStoreSubscribe from './actions/useApplifyDoDataStoreSubscribe'
import useApplifyDoDataStoreUnsubscribe from './actions/useApplifyDoDataStoreUnsubscribe'
import useApplifyDoForgotPassword from './actions/useApplifyDoForgotPassword'
import useApplifyDoForgotPasswordSubmit from './actions/useApplifyDoForgotPasswordSubmit'
import useApplifyDoResendVerificationCode from './actions/useApplifyDoResendVerificationCode'
import useApplifyDoSignIn from './actions/useApplifyDoSignIn'
import useApplifyDoSignInFacebook from './actions/useApplifyDoSignInFacebook'
import useApplifyDoSignInGoogle from './actions/useApplifyDoSignInGoogle'
import useApplifyDoSignOut from './actions/useApplifyDoSignOut'
import useApplifyDoSignUp from './actions/useApplifyDoSignUp'
import useApplifyDoSignUpFacebook from './actions/useApplifyDoSignUpFacebook'
import useApplifyDoSignUpGoogle from './actions/useApplifyDoSignUpGoogle'
import useApplifyDoSwitchTenant from './actions/useApplifyDoSwitchTenant'
import useApplifyDoUserDetailsRefresh from './actions/useApplifyDoUserDetailsRefresh'
import useApplifyEffects from './actions/useApplifyEffects'
import useApplifyIsUserNotConfirmedError from './actions/useApplifyIsUserNotConfirmedError'
import useApplifyUserDetailsCatch from './actions/useApplifyUserDetailsCatch'
import useApplifyUserDetailsFinally from './actions/useApplifyUserDetailsFinally'
import useApplifyUserDetailsLoad from './actions/useApplifyUserDetailsLoad'
import useApplifyUserDetailsSet from './actions/useApplifyUserDetailsSet'
export interface ApplifyControllerProps {
	children: any
	env: ApplifyEnv
}

export function ApplifyController(props: ApplifyControllerProps) {
	const { children, env } = props

	// STATE
	const _s = useCtrlState<ApplifyData>({
		_busy: false,
		_session: undefined,
		_roles: undefined,
		_subscriptions: [],
		initialized: false,
		authenticated: false,
		waitForSynced: true,
		user: undefined,
		signupCode: undefined,
		tenantId: undefined,
		initStamp: DateTime.now(),
		env,
	})

	// ACTIONS
	const _a = useRef<ApplifyActions>({} as ApplifyActions)
	_a.current.doSetSignupCode = useCtrlStateSetter(_s, 'signupCode')
	_a.current._userDetailsLoad = useApplifyUserDetailsLoad(_s, _a)
	_a.current._userDetailsSet = useApplifyUserDetailsSet(_s, _a)
	_a.current._userDetailsCatch = useApplifyUserDetailsCatch(_s, _a)
	_a.current._userDetailsFinally = useApplifyUserDetailsFinally(_s, _a)
	_a.current.isUserNotConfirmedError = useApplifyIsUserNotConfirmedError(_s, _a)
	_a.current.doClearDb = useApplifyDoClearDb(_s, _a)
	_a.current.doSignInGoogle = useApplifyDoSignInGoogle(_s, _a)
	_a.current.doSignInFacebook = useApplifyDoSignInFacebook(_s, _a)
	_a.current.doSignUpGoogle = useApplifyDoSignUpGoogle(_s, _a)
	_a.current.doSignUpFacebook = useApplifyDoSignUpFacebook(_s, _a)
	_a.current.doSignIn = useApplifyDoSignIn(_s, _a)
	_a.current.doSignOut = useApplifyDoSignOut(_s, _a)
	_a.current.doSignUp = useApplifyDoSignUp(_s, _a)
	_a.current.doConfirmSignUp = useApplifyDoConfirmSignUp(_s, _a)
	_a.current.doConfirmSignIn = useApplifyDoConfirmSignIn(_s, _a)
	_a.current.doForgotPassword = useApplifyDoForgotPassword(_s, _a)
	_a.current.doForgotPasswordSubmit = useApplifyDoForgotPasswordSubmit(_s, _a)
	_a.current.doResendVerificationCode = useApplifyDoResendVerificationCode(_s, _a)
	_a.current.doUserDetailsRefresh = useApplifyDoUserDetailsRefresh(_s, _a)
	_a.current.doSwitchTenant = useApplifyDoSwitchTenant(_s, _a)
	_a.current.doDataStoreSubscribe = useApplifyDoDataStoreSubscribe(_s, _a)
	_a.current.doDataStoreUnsubscribe = useApplifyDoDataStoreUnsubscribe(_s, _a)

	// INITIALIZE
	useApplifyEffects(_s, _a)

	// CTX
	const ctx = useCtrlCtx(_s)

	// ACTIONS CTX
	const actionsCtx = useCtrlActionsCtx(_a, 'Applify')

	return (
		<ApplifyContext.Provider value={ctx}>
			<ApplifyActionsContext.Provider value={actionsCtx}>
				{children}
			</ApplifyActionsContext.Provider>
		</ApplifyContext.Provider>
	)
}
export default ApplifyController

export interface ApplifyData {
	_busy: boolean
	_session: any
	_roles: RolesManager
	_subscriptions: any[]
	initialized: boolean
	authenticated: boolean
	waitForSynced: boolean
	user: {
		userId: string
		username: string
		attributes: any
	}
	signupCode: string
	tenantId: string
	initStamp: DateTime
	env: ApplifyEnv
}
export type ApplifyState = CtrlState<ApplifyData>
export const ApplifyContext = React.createContext<ApplifyData>(null)

export interface ApplifyActions {
	_userDetailsSet: () => void
	_userDetailsCatch: () => void
	_userDetailsFinally: () => void
	_userDetailsLoad: () => void
	isUserNotConfirmedError: (err: Error) => boolean
	doClearDb: () => Promise<void>
	doUserDetailsRefresh: () => Promise<void>
	doSwitchTenant: (tenantId: string) => Promise<void>
	doSignInFacebook: () => void
	doSignInGoogle: () => void
	doSignUpFacebook: () => void
	doSignUpGoogle: () => void
	doSetSignupCode: (v: string) => void
	doSignIn: (email: string, password: string) => Promise<void>
	doSignUp: (email: string, password: string) => Promise<SignUpOutput>
	doResendVerificationCode: (email: string) => any
	doForgotPassword: (email: string) => Promise<void>
	doForgotPasswordSubmit: (email: string, code: string, pword: string) => Promise<void>
	doConfirmSignUp: (email: string, code: string) => Promise<void>
	doConfirmSignIn: (email: string, code: string) => Promise<void>
	doSignOut: () => Promise<void>
	doDataStoreSubscribe: (subscription: any) => any
	doDataStoreUnsubscribe: (subscription: any) => void
}
export type ApplifyActionsState = MutableRefObject<ApplifyActions>
export const ApplifyActionsContext = React.createContext<ApplifyActions>(null)

export const useApplifyActions = () => useContext(ApplifyActionsContext)

export function useApplifyTenantId() {
	const { tenantId } = useContext(ApplifyContext)
	return useMemo(() => {
		// TODO - remove this temporary code
		if (tenantId === 'triggerignore_tenantA') return 'myCo'
		if (tenantId === 'tenantB') return 'myOtherCo'
		//---------------------
		return tenantId
	}, [tenantId])
}

export function useApplifyUserName() {
	const { user } = useContext(ApplifyContext)
	return useMemo(() => (user ? user['attributes']['name'] : ''), [user])
}

export function useApplifyUserRoles(): RolesManager {
	const { user } = useContext(ApplifyContext)
	return useMemo(
		() => RolesManager.fromCommaList(user ? user['attributes']['custom:roles'] : ''),
		[user]
	)
}

export function useApplifyCtxChanged() {
	const { user, tenantId, initStamp } = useContext(ApplifyContext)
	return useMemo(
		() => DateTime.now(),
		// eslint-disable-next-line react-hooks/exhaustive-deps
		[user, tenantId, initStamp]
	)
}

export function useApplifyTenantTitle() {
	const { user } = useContext(ApplifyContext)
	return useMemo(() => (user ? user['attributes']['custom:tenanttitle'] : ''), [user])
}

export function useApplifyEnv() {
	const { env } = useContext(ApplifyContext)
	return useMemo(() => env, [env])
}

export function useApplifySignupCode() {
	const { signupCode } = useContext(ApplifyContext)
	return useMemo(() => signupCode, [signupCode])
}

export function useApplifyWaitForSynced() {
	const { waitForSynced } = useContext(ApplifyContext)
	return useMemo(() => waitForSynced, [waitForSynced])
}

export function useApplifyAuthenticated() {
	const { authenticated } = useContext(ApplifyContext)
	return useMemo(() => authenticated, [authenticated])
}

export function useApplifyInitialized() {
	const { initialized } = useContext(ApplifyContext)
	return useMemo(() => initialized, [initialized])
}
