import Logger from '@thats/utils/log/Logger'
import useI18n from '@thatweb/core-i18n/hooks/useI18n'
import { usePleaseWaitActions } from '@thatweb/pleasewait/controllers/PleaseWaitController'
import { DataStore } from 'aws-amplify/datastore'
import { useCallback } from 'react'
import apiSelfSwitchTenant from '../../app/svcs/apiSelfSwitchTenant'
import { ApplifyActionsState, ApplifyState } from '../ApplifyController'

export default useApplifyDoSwitchTenant

export function useApplifyDoSwitchTenant(
	{ current: _s }: ApplifyState,
	{ current: _a }: ApplifyActionsState
) {
	const { doWaitFor, doStopWaitFor } = usePleaseWaitActions()
	const i18n = useI18n()

	return useCallback(
		(tId: string) => {
			return new Promise<void>((resolve, reject) =>
				doWaitFor('switching', i18n.raw('switching-tenant'), () =>
					apiSelfSwitchTenant({ tenantId: tId })
						.then(() => {
							Logger.info('Stopping Datastore')
							return DataStore.stop()
						})
						.then(() => {
							Logger.info('Clearing Datastore')
							return DataStore.clear()
						})
						.then(_a.doUserDetailsRefresh)
						.then(() => {
							Logger.info('Starting Datastore')
							return DataStore.start()
						})
						.then(resolve)
						.catch(reject)
						.finally(() => {
							doStopWaitFor('switching')
						})
				)
			)
		},
		[_a, doStopWaitFor, doWaitFor, i18n]
	)
}
