import Logger from '@thats/utils/log/Logger'
import { DataStore } from 'aws-amplify/datastore'
import { Hub } from 'aws-amplify/utils'
import { useCallback, useEffect } from 'react'
import { ApplifyActionsState, ApplifyState } from '../ApplifyController'

export default useApplifyEffects

export function useApplifyEffects(
	{ current: _s }: ApplifyState,
	{ current: _a }: ApplifyActionsState
) {
	// AUTH EVENT Handlers
	const listener = useCallback(
		data => {
			switch (data.payload.event) {
				case 'signedIn':
				case 'signedUp':
				case 'signedOut':
				case 'tokenRefresh':
					return void _a._userDetailsLoad()
				default: {
					Logger.info('useApplifyEffects: unhandled event', data)
					return undefined
				}
			}
		},
		[_a]
	)

	useEffect(() => {
		DataStore.configure({})

		const listenCancel = Hub.listen('auth', listener)
		void _a._userDetailsLoad()
		return () => listenCancel()
	}, [listener, _a])
}
