"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDataStoreGet = void 0;
const useDebounce_1 = __importDefault(require("@thatreact/core-hooks/hooks/useDebounce"));
const Logger_1 = __importDefault(require("@thats/utils/log/Logger"));
const datastore_1 = require("aws-amplify/datastore");
const react_1 = require("react");
const ApplifyController_1 = require("../controllers/ApplifyController");
exports.default = useDataStoreGet;
function useDataStoreGet(PM, id, o = {}) {
    const { skip = false, log = false } = o;
    const subscription = (0, react_1.useRef)();
    const waitForSynced = (0, ApplifyController_1.useApplifyWaitForSynced)();
    const tenantId = (0, ApplifyController_1.useApplifyTenantId)();
    const ctxChanged = (0, ApplifyController_1.useApplifyCtxChanged)();
    const { doDataStoreSubscribe, doDataStoreUnsubscribe } = (0, ApplifyController_1.useApplifyActions)();
    const [result, setResult] = (0, react_1.useState)({
        _: undefined,
        isLoading: !skip,
        isSynced: false,
    });
    const doSetResult = (0, useDebounce_1.default)(newResult => {
        setResult(newResult);
    }, []);
    (0, react_1.useEffect)(() => {
        if (skip || !id)
            return;
        if (subscription.current)
            doDataStoreUnsubscribe(subscription.current);
        subscription.current = doDataStoreSubscribe(datastore_1.DataStore.observeQuery(PM, (p) => p.tenantId.eq(tenantId) && p.id.eq(id)).subscribe(snapshot => {
            var _a;
            const { items, isSynced } = snapshot;
            if (log)
                Logger_1.default.info('Snapshot Received', { items, isSynced });
            if (!waitForSynced || isSynced)
                doSetResult({
                    _: (_a = items === null || items === void 0 ? void 0 : items[0]) !== null && _a !== void 0 ? _a : undefined,
                    isLoading: false,
                    isSynced,
                });
        }));
        return () => {
            if (subscription.current)
                doDataStoreUnsubscribe(subscription.current);
        };
    }, [
        skip,
        PM,
        tenantId,
        id,
        log,
        waitForSynced,
        doSetResult,
        ctxChanged,
        doDataStoreUnsubscribe,
        doDataStoreSubscribe,
    ]);
    (0, react_1.useEffect)(() => {
        doSetResult({
            _: undefined,
            isLoading: !skip,
            isSynced: false,
        });
    }, [ctxChanged, doSetResult, skip]);
    return result;
}
exports.useDataStoreGet = useDataStoreGet;
//# sourceMappingURL=useDataStoreGet.js.map