"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDataStoreList = void 0;
const useDebounce_1 = require("@thatreact/core-hooks/hooks/useDebounce");
const datastore_1 = require("aws-amplify/datastore");
const react_1 = require("react");
const ApplifyController_1 = require("../controllers/ApplifyController");
exports.default = useDataStoreList;
function useDataStoreList(PM, criteria, skip = false) {
    const ctxChanged = (0, ApplifyController_1.useApplifyCtxChanged)();
    const subscription = (0, react_1.useRef)();
    const { doDataStoreSubscribe, doDataStoreUnsubscribe } = (0, ApplifyController_1.useApplifyActions)();
    const [result, setResult] = (0, react_1.useState)({
        _: [],
        isLoading: !skip,
        isSynced: false,
    });
    const doSetResult = (0, useDebounce_1.useDebounceWithLeading)(newResult => setResult(newResult), []);
    (0, react_1.useEffect)(() => {
        if (skip)
            return;
        if (subscription.current)
            doDataStoreUnsubscribe(subscription.current);
        subscription.current = doDataStoreSubscribe(datastore_1.DataStore.observeQuery(PM, criteria).subscribe(snapshot => {
            const { items, isSynced } = snapshot;
            doSetResult({
                _: items,
                isLoading: false,
                isSynced,
            });
        }));
        return () => {
            if (subscription.current)
                doDataStoreUnsubscribe(subscription.current);
        };
    }, [skip, PM, criteria, doSetResult, doDataStoreUnsubscribe, doDataStoreSubscribe, ctxChanged]);
    (0, react_1.useEffect)(() => {
        doSetResult({
            _: [],
            isLoading: !skip,
            isSynced: false,
        });
    }, [ctxChanged, doSetResult, skip]);
    return result;
}
exports.useDataStoreList = useDataStoreList;
//# sourceMappingURL=useDataStoreList.js.map