import { useDebounceWithLeading } from '@thatreact/core-hooks/hooks/useDebounce'
import {
	DataStore,
	PersistentModel,
	PersistentModelConstructor,
	Predicates,
	RecursiveModelPredicateExtender,
} from 'aws-amplify/datastore'
import { useEffect, useRef, useState } from 'react'
import { useApplifyActions, useApplifyCtxChanged } from '../controllers/ApplifyController'

export default useDataStoreList

export type UseDataStoreListResult<PM extends PersistentModel> = {
	_: PM[]
	isLoading: boolean
	isSynced: boolean
}

export function useDataStoreList<PM extends PersistentModel>(
	PM: PersistentModelConstructor<PM>,
	criteria?: RecursiveModelPredicateExtender<PM> | typeof Predicates.ALL,
	skip: boolean = false
): UseDataStoreListResult<PM> {
	const ctxChanged = useApplifyCtxChanged()
	const subscription = useRef<any>()
	const { doDataStoreSubscribe, doDataStoreUnsubscribe } = useApplifyActions()
	const [result, setResult] = useState<UseDataStoreListResult<PM>>({
		_: [],
		isLoading: !skip,
		isSynced: false,
	})

	const doSetResult = useDebounceWithLeading(newResult => setResult(newResult), [])

	useEffect(() => {
		if (skip) return
		if (subscription.current) doDataStoreUnsubscribe(subscription.current)
		subscription.current = doDataStoreSubscribe(
			DataStore.observeQuery(PM, criteria).subscribe(snapshot => {
				const { items, isSynced } = snapshot
				doSetResult({
					_: items,
					isLoading: false,
					isSynced,
				})
			})
		)
		return () => {
			if (subscription.current) doDataStoreUnsubscribe(subscription.current)
		}
	}, [skip, PM, criteria, doSetResult, doDataStoreUnsubscribe, doDataStoreSubscribe, ctxChanged])

	useEffect(() => {
		doSetResult({
			_: [],
			isLoading: !skip,
			isSynced: false,
		})
	}, [ctxChanged, doSetResult, skip])

	return result
}
