import doNothing from '@thats/utils/misc/doNothing'
import { useCallback, useEffect, useState } from 'react'

export interface GQLResult<T> {
	data: T
	error: Error
	loading: boolean
	refresh: () => Promise<void>
}

export function useGQLMutation<T>(api: () => Promise<T>): GQLResult<T> {
	const [initiated, setInitiated] = useState(false)
	const [data, setData] = useState<T>(undefined)
	const [error, setError] = useState<Error>(undefined)
	const [loading, setLoading] = useState(true)

	const refresh = useCallback(async () => {
		setLoading(true)
		setError(undefined)
		setData(undefined)
		try {
			setData(await api())
		} catch (e) {
			setError(e)
		}
		setLoading(false)
	}, [api])

	useEffect(() => {
		if (initiated) return
		setInitiated(true)
		refresh().catch(doNothing)
	}, [initiated, refresh])

	return { data, error, loading: !initiated, refresh }
}
