import doNothing from '@thats/utils/misc/doNothing'
import { useCallback, useEffect, useState } from 'react'

export interface GQLResult<T> {
	data: T
	error: Error
	loading: boolean
	refresh: () => Promise<void>
}

export function useGQLQuery<T>(api: () => Promise<T>, key: string = 'gqlquery'): GQLResult<T> {
	const [data, setData] = useState<T>(undefined)
	const [error, setError] = useState<Error>(undefined)
	const [loading, setLoading] = useState(true)

	const refresh = useCallback(async () => {
		setLoading(true)
		setError(undefined)
		setData(undefined)
		try {
			setData(await api())
		} catch (e) {
			setError(e)
		}
		setLoading(false)
	}, [api])

	useEffect(() => {
		refresh().catch(doNothing)
	}, [key])

	return { data, error, loading, refresh }
}

export default useGQLQuery
