import QueryOptions from '@thatamplify/core/api/query/QueryOptions'
import QueryResponse from '@thatamplify/core/api/query/QueryResponse'
import QueryResponseKind from '@thatamplify/core/api/query/QueryResponseKind'
import Type from '@thatamplify/core/app/_/Type'
import Logger from '@thats/utils/log/Logger'
import { generateClient } from 'aws-amplify/api'

export async function gqlQuery<R extends QueryResponseKind<T> = boolean, T extends Type = any>(
	query: string,
	variables?: any,
	o: QueryOptions = {}
): Promise<QueryResponse<R, T>> {
	const client = generateClient()

	// Get options
	const { log = false } = o

	// Process the question
	let result

	if (log) Logger.info('Query Request', { query, variables })

	// Perform the query
	try {
		result = await client.graphql({ query, variables })
	} catch (e) {
		if (e.errors) {
			result = e
		} else {
			result = { data: undefined, errors: [e] }
		}
	}

	// Log the result
	if (log) Logger.info('Query Response', result)

	// Return the Result
	return result
}
export default gqlQuery
